@@ -23,12 +23,13 @@ or with a python-like shorthand
 
 - catherinedevlin.blogspot.com  May 31, 2006
 """
-import sys, os, re, sqlpython, cx_Oracle, pyparsing, re, completion, datetime, pickle, binascii, subprocess
+import sys, os, re, sqlpython, cx_Oracle, pyparsing, re, completion, datetime, pickle, binascii, subprocess, time, itertools, hashlib
 from cmd2 import Cmd, make_option, options, Statekeeper, Cmd2TestCase
 from output_templates import output_templates
 from metadata import metaqueries
 from plothandler import Plot
 from sqlpython import Parser
+import imagedetect
 import warnings
 warnings.filterwarnings('ignore', 'BaseException.message', DeprecationWarning)
 try:
@@ -81,7 +82,7 @@ WHERE atc.table_name = :object_name
 AND      atc.owner = :owner
 ORDER BY atc.column_id;""",
 'oneColComments': """
-SELECTatc.column_name,
+SELECT atc.column_name,
 acc.comments             
 FROM all_tab_columns atc
 JOIN all_col_comments acc ON (atc.owner = acc.owner and atc.table_name = acc.table_name and atc.column_name = acc.column_name)
@@ -281,6 +282,48 @@ def offset_to_line(instring, offset):
             return lineNum, line, offset
         lineNum += 1
         offset -= len(line)
+
+class BlobDisplayer(object):
+    folder_name = 'sqlpython_blob_store'
+    imgwidth = 400
+    def folder_ok(self):
+        if not os.access(self.folder_name, os.F_OK):
+            try:
+                os.mkdir(self.folder_name)
+                readme = open(os.path.join(self.folder_name, 'README.txt'),'w')
+                readme.write('''
+                                Temporary files for display of BLOBs from within
+                                sqlpython.  Delete when sqlpython is closed.''')
+                readme.close()
+            except:
+                return False
+        return True
+    def __init__(self, blob, under_limit):
+        self.url = ''
+        if under_limit:
+            self.blob = blob.read()
+            self.hashed = hashlib.md5(self.blob).hexdigest()
+            self.extension = imagedetect.extension_from_data(self.blob)
+            if self.folder_ok():
+                self.file_name = '%s/%s%s' % (
+                    os.path.join(os.getcwd(), self.folder_name), 
+                    self.hashed, self.extension)
+                self.url = 'file://%s' % self.file_name
+                if not os.access(self.file_name, os.F_OK):
+                    outfile = open(self.file_name, 'wb')
+                    outfile.write(self.blob)
+                    outfile.close()
+    def __str__(self):
+        if self.url:
+            return '(BLOB at %s)' % self.url
+        else:
+            return '(BLOB)'
+    def html(self):
+        if self.url:
+            return '<a href="%s"><img src="%s" width="%d" /></a>' % (
+                self.url, self.url, self.imgwidth)
+        else:
+            return '(BLOB not saved, check bloblimit)'
         
 class sqlpyPlus(sqlpython.sqlpython):
     defaultExtension = 'sql'
@@ -304,7 +347,7 @@ class sqlpyPlus(sqlpython.sqlpython):
     def __init__(self):
         sqlpython.sqlpython.__init__(self)
         self.binds = CaselessDict()
-        self.settable += 'autobind colors commit_on_exit maxfetch maxtselctrows rows_remembered scan serveroutput sql_echo timeout heading wildsql'.split()
+        self.settable += 'autobind bloblimit colors commit_on_exit maxfetch maxtselctrows rows_remembered scan serveroutput sql_echo timeout heading wildsql version'.split()
         self.settable.remove('case_insensitive')
         self.settable.sort()
         self.stdoutBeforeSpool = sys.stdout
@@ -319,6 +362,8 @@ class sqlpyPlus(sqlpython.sqlpython):
         self.substvars = {}
         self.result_history = []
         self.rows_remembered = 10000
+        self.bloblimit = 5
+        self.version = 'SQLPython %s' % sqlpython.__version__
         self.pystate = {'r': [], 'binds': self.binds, 'substs': self.substvars}
         
     # overrides cmd's parseline
@@ -360,7 +405,9 @@ class sqlpyPlus(sqlpython.sqlpython):
         
     def postcmd(self, stop, line):
         """Hook method executed just after a command dispatch is finished."""        
-        if (self.rdbms == 'oracle') and self.serveroutput:
+        if (    hasattr(self, 'rdbms') 
+            and (self.rdbms == 'oracle') 
+            and self.serveroutput):
             self.dbms_output()
         return stop
     
@@ -449,7 +496,6 @@ class sqlpyPlus(sqlpython.sqlpython):
         self.colnames = [d[0] for d in self.curs.description]
         if outformat in output_templates:
             self.colnamelen = max(len(colname) for colname in self.colnames)
-            self.coltypes = [d[1] for d in self.curs.description]
             result = output_templates[outformat].generate(formattedForSql=self.formattedForSql, **self.__dict__)        
         elif outformat == '\\t': # transposed
             rows = [self.colnames]
@@ -668,7 +714,7 @@ class sqlpyPlus(sqlpython.sqlpython):
                 return
             total_len -= len(rset)
             self.pystate['r'][i] = []
-            
+        
     def do_select(self, arg, bindVarsIn=None, terminator=None):
         """Fetch rows from a table.
 
@@ -698,6 +744,14 @@ class sqlpyPlus(sqlpython.sqlpython):
         else: # this is an ugly workaround for the evil paramstyle curse upon DB-API2
             self.curs.execute(self.querytext)
         self.rows = self.curs.fetchmany(min(self.maxfetch, (rowlimit or self.maxfetch)))
+        self.coltypes = [d[1] for d in self.curs.description]
+        if cx_Oracle.BLOB in self.coltypes:
+            self.rows = [
+                 [(    (coltype == cx_Oracle.BLOB) 
+                   and BlobDisplayer(datum, (rownum < self.bloblimit)))
+                   or datum
+                   for (datum, coltype) in zip(row, self.coltypes)]
+                 for (rownum, row) in enumerate(self.rows)]
         self.rc = len(self.rows)
         if self.rc != 0:
             resultset = ResultSet()
@@ -828,13 +882,15 @@ class sqlpyPlus(sqlpython.sqlpython):
                            value FROM v$parameter WHERE name LIKE '%%%s%%';""" % paramname)
         else:
             argpieces = arg.lower().split()
-            flagless_argpieces = [a for a in argpieces if not a.startswith('-')]
-            for (kwd, shortcut) in (('ind', '\\di'), ('schema', '\\dn'), ('tablesp', '\\db'), 
-                                    ('trig', '\\dt'), ('view', '\\dv'), ('cons', '\\dc'),
-                                    ('comm', '\\dd'), ('ref', 'ref')):
-                if flagless_argpieces[0].lower().startswith(kwd):
-                    return self._show_shortcut(shortcut, argpieces)
+            argpieces = [a for a in argpieces if not a.startswith('-')]
             try:
+                for (kwd, shortcut) in (
+                        ('ind', '\\di'), ('schema', '\\dn'), 
+                        ('tablesp', '\\db'), ('trig', '\\dt'), 
+                        ('view', '\\dv'), ('cons', '\\dc'),
+                        ('comm', '\\dd'), ('ref', 'ref')):
+                    if argpieces[0].lower().startswith(kwd):
+                        return self._show_shortcut(shortcut, argpieces)
                 if argpieces[0][:3] == 'err':
                     return self._show_errors(all_users=False, limit=1, targets=argpieces[1:])
                 elif (argpieces[0], argpieces[1][:3]) == ('all','err'):
